@@ -112,7 +112,7 @@ StringXString GeneralOptions_String[] = {
 #elif defined(__APPLE__)
     "open -t %s" ,
 #else
-    "sensible-editor '%s'" ,
+    "gedit %s" ,
 #endif
     "System command to launch a text editor" },
   { F|S, "TmpFileName" , opt_general_tmp_filename ,
@@ -1309,7 +1309,8 @@ StringXNumber SolverOptions_Number[] = {
   { F|O, "AutoShowViews" , opt_solver_auto_show_views , 2. ,
     "Automcatically show newly merged results (0: none; 1: all; 2: last one)" },
   { F|O, "AutoShowLastStep" , opt_solver_auto_show_last_step , 1. ,
-    "Automatically show the last time step in newly merged results" },
+    "Automatically show the last step in newly merged results, if there are "
+    "more than 2 steps" },
 
   { F|O, "Plugins" , opt_solver_plugins , 0. ,
     "Enable default solver plugins?" },
@@ -1609,8 +1610,8 @@ StringXNumber ViewOptions_Number[] = {
   { F|O, "ShowScale" , opt_view_show_scale , 1. ,
     "Show value scale?" },
   { F|O, "ShowTime" , opt_view_show_time , 3. ,
-    "Time display mode (0=hidden, 1=time value if multi-step, 2=time value always, "
-    "3=time step if multi-step, 4=time step always)" },
+    "Time display mode (0=none, 1=time series, 2=harmonic data, 3=automatic, "
+    "4=step data, 5=multi-step data, 6=real eigenvalues, 7=complex eigenvalues)" },
   { F|O, "SmoothNormals" , opt_view_smooth_normals , 0. ,
     "Smooth the normals?" },
   { F|O, "Stipple" , opt_view_use_stipple , 0. ,
