@@ -84,8 +84,9 @@ int GmshInitialize(int argc, char **argv)
   PluginManager::instance()->registerDefaultPlugins();
 #endif
 
-  // Initialize robust predicates (no static filter for now, we do not know the size of the domain)
-  robustPredicates::exactinit(0,1.0,1.0,1.0);
+  // Initialize robust predicates (no static filter for now, we do not know the
+  // size of the domain)
+  robustPredicates::exactinit(0, 1.0, 1.0, 1.0);
 
   if(dummy) delete dummy;
   return 1;
@@ -113,62 +114,71 @@ int GmshSetBoundingBox(double xmin, double xmax,
 int GmshSetOption(const std::string &category, const std::string &name,
                   std::string value, int index)
 {
-  return StringOption(GMSH_SET|GMSH_GUI, category.c_str(), index, name.c_str(), value);
+  return StringOption(GMSH_SET|GMSH_GUI, category.c_str(), index, name.c_str(),
+                      value, false);
 }
 
 int GmshSetOption(const std::string &category, const std::string &name,
                   double value, int index)
 {
-  return NumberOption(GMSH_SET|GMSH_GUI, category.c_str(), index, name.c_str(), value);
+  return NumberOption(GMSH_SET|GMSH_GUI, category.c_str(), index, name.c_str(),
+                      value, false);
 }
 
 int GmshSetOption(const std::string &category, const std::string &name,
                   unsigned int value, int index)
 {
-  return ColorOption(GMSH_SET|GMSH_GUI, category.c_str(), index, name.c_str(), value);
+  return ColorOption(GMSH_SET|GMSH_GUI, category.c_str(), index, name.c_str(),
+                     value, false);
 }
 
 void GmshSetStringOption(const std::string &category, const std::string &name,
                          std::string value, int index)
 {
-  StringOption(GMSH_SET|GMSH_GUI, category.c_str(), index, name.c_str(), value);
+  StringOption(GMSH_SET|GMSH_GUI, category.c_str(), index, name.c_str(),
+               value, false);
 }
 
 void GmshSetNumberOption(const std::string &category, const std::string &name,
                          double value, int index)
 {
-  NumberOption(GMSH_SET|GMSH_GUI, category.c_str(), index, name.c_str(), value);
+  NumberOption(GMSH_SET|GMSH_GUI, category.c_str(), index, name.c_str(),
+               value, false);
 }
 
 void GmshSetColorOption(const std::string &category, const std::string &name,
                         unsigned int value, int index)
 {
-  ColorOption(GMSH_SET|GMSH_GUI, category.c_str(), index, name.c_str(), value);
+  ColorOption(GMSH_SET|GMSH_GUI, category.c_str(), index, name.c_str(),
+              value, false);
 }
 
 int GmshGetOption(const std::string &category, const std::string &name,
                   std::string &value, int index)
 {
-  return StringOption(GMSH_GET, category.c_str(), index, name.c_str(), value);
+  return StringOption(GMSH_GET, category.c_str(), index, name.c_str(), value,
+                      false);
 }
 
 int GmshGetOption(const std::string &category, const std::string &name,
                   double &value, int index)
 {
-  return NumberOption(GMSH_GET, category.c_str(), index, name.c_str(), value);
+  return NumberOption(GMSH_GET, category.c_str(), index, name.c_str(), value,
+                      false);
 }
 
 int GmshGetOption(const std::string &category, const std::string &name,
                   unsigned int &value, int index)
 {
-  return ColorOption(GMSH_GET, category.c_str(), index, name.c_str(), value);
+  return ColorOption(GMSH_GET, category.c_str(), index, name.c_str(), value,
+                     false);
 }
 
 std::string GmshGetStringOption(const std::string &category, const std::string &name,
                                 int index)
 {
   std::string value;
-  StringOption(GMSH_GET, category.c_str(), index, name.c_str(), value);
+  StringOption(GMSH_GET, category.c_str(), index, name.c_str(), value, false);
   return value;
 }
 
@@ -176,7 +186,7 @@ double GmshGetNumberOption(const std::string &category, const std::string &name,
                            int index)
 {
   double value;
-  NumberOption(GMSH_GET, category.c_str(), index, name.c_str(), value);
+  NumberOption(GMSH_GET, category.c_str(), index, name.c_str(), value, false);
   return value;
 }
 
@@ -184,7 +194,7 @@ int GmshGetColorOption(const std::string &category, const std::string &name,
                        int index)
 {
   unsigned int value;
-  ColorOption(GMSH_GET, category.c_str(), index, name.c_str(), value);
+  ColorOption(GMSH_GET, category.c_str(), index, name.c_str(), value, false);
   return value;
 }
 
