@@ -51,6 +51,8 @@ int Msg::_verbosity = 5;
 int Msg::_progressMeterStep = 20;
 int Msg::_progressMeterCurrent = 0;
 std::map<std::string, double> Msg::_timers;
+bool Msg::_infoCpu = false;
+double Msg::_startTime = 0.;
 int Msg::_warningCount = 0;
 int Msg::_errorCount = 0;
 int Msg::_atLeastOneErrorInRun = 0;
@@ -106,6 +108,7 @@ static void addGmshPathToEnvironmentVar(const std::string &name)
 
 void Msg::Init(int argc, char **argv)
 {
+  _startTime = TimeOfDay();
 #if defined(HAVE_MPI)
   int flag;
   MPI_Initialized(&flag);
@@ -252,6 +255,52 @@ static int streamIsVT100(FILE* stream)
   return 1;
 }
 
+std::string Msg::PrintResources(bool printDate, bool printWallTime,
+                                bool printCpu, bool printMem)
+{
+  long mem = GetMemoryUsage();
+
+  std::string pdate = "";
+  if(printDate){
+    time_t now;
+    time(&now);
+    pdate = ctime(&now);
+    pdate.resize(pdate.size() - 1);
+    if(printWallTime || printCpu || (printMem && mem))
+      pdate += ", ";
+  }
+
+  std::string pwall = "";
+  if(printWallTime){
+    char tmp[128];
+    sprintf(tmp, "Wall = %gs", TimeOfDay() - _startTime);
+    pwall = tmp;
+    if(printCpu || (printMem && mem))
+      pwall += ", ";
+  }
+
+  std::string pcpu = "";
+  if(printCpu){
+    char tmp[128];
+    sprintf(tmp, "CPU = %gs", Cpu());
+    pcpu = tmp;
+    if(printMem && mem)
+      pcpu += ", ";
+  }
+
+  std::string pmem = "";
+  if(mem && printMem){
+    char tmp[128];
+    sprintf(tmp, "Mem = %gMb", (double)mem / 1024. / 1024.);
+    pmem = tmp;
+  }
+
+  std::string str;
+  if(pdate.size() || pwall.size() || pcpu.size() || pmem.size())
+    str += " (" + pdate +  pwall +  pcpu +  pmem + ")";
+  return str;
+}
+
 void Msg::Fatal(const char *fmt, ...)
 {
   _errorCount++;
@@ -388,6 +437,11 @@ void Msg::Info(const char *fmt, ...)
   vsnprintf(str, sizeof(str), fmt, args);
   va_end(args);
 
+  if(_infoCpu){
+    std::string res = PrintResources(false, true, true, true);
+    strcat(str, res.c_str());
+  }
+
   if(_callback) (*_callback)("Info", str);
   if(_client) _client->Info(str);
 
@@ -468,6 +522,11 @@ void Msg::StatusBar(bool log, const char *fmt, ...)
   vsnprintf(str, sizeof(str), fmt, args);
   va_end(args);
 
+  if(_infoCpu){
+    std::string res = PrintResources(false, true, true, true);
+    strcat(str, res.c_str());
+  }
+
   if(_callback && log) (*_callback)("Info", str);
   if(_client && log) _client->Info(str);
 
@@ -787,6 +846,44 @@ void Msg::SetOnelabString(std::string name, std::string val, bool visible,
 #endif
 }
 
+void Msg::AddOnelabStringChoice(std::string name, std::string kind,
+                                std::string value, bool updateValue,
+                                bool readOnly, bool visible)
+{
+#if defined(HAVE_ONELAB)
+  if(_onelabClient){
+    std::vector<std::string> choices;
+    std::vector<onelab::string> ps;
+    _onelabClient->get(ps, name);
+    if(ps.size()){
+      choices = ps[0].getChoices();
+      if(std::find(choices.begin(), choices.end(), value) == choices.end())
+        choices.push_back(value);
+      if(updateValue)
+        ps[0].setValue(value);
+    }
+    else{
+      ps.resize(1);
+      ps[0].setName(name);
+      ps[0].setKind(kind);
+      ps[0].setValue(value);
+      choices.push_back(value);
+    }
+    ps[0].setChoices(choices);
+    if(readOnly){
+      ps[0].setReadOnly(true);
+      ps[0].setAttribute("AutoCheck", "0");
+    }
+    else{
+      ps[0].setReadOnly(false);
+      ps[0].setAttribute("AutoCheck", "1");
+    }
+    ps[0].setVisible(visible);
+    _onelabClient->set(ps[0]);
+  }
+#endif
+}
+
 double Msg::GetOnelabNumber(std::string name, double defaultValue,
                             bool errorIfMissing)
 {
@@ -886,7 +983,8 @@ void Msg::InitializeOnelab(const std::string &name, const std::string &sockname)
     std::vector<onelab::string> ps;
     _onelabClient->get(ps, name + "/Action");
     if(ps.size()){
-      Info("Performing ONELAB '%s'", ps[0].getValue().c_str());
+      //removed message, as terminal is set to 1 when we get here
+      //Info("Performing ONELAB '%s'", ps[0].getValue().c_str());
       if(ps[0].getValue() == "initialize") Exit(0);
     }
   }
@@ -952,6 +1050,14 @@ void Msg::LoadOnelabClient(const std::string &clientName, const std::string &soc
 #endif
 }
 
+int Msg::GetNumOnelabClients()
+{
+#if defined(HAVE_ONELAB)
+  return onelab::server::instance()->getNumClients();
+#endif
+  return 0;
+}
+
 #if defined(HAVE_ONELAB)
 static void _setStandardOptions(onelab::parameter *p,
                                 std::map<std::string, std::vector<double> > &fopt,
@@ -965,6 +1071,7 @@ static void _setStandardOptions(onelab::parameter *p,
   if(copt.count("Highlight")) p->setAttribute("Highlight", copt["Highlight"][0]);
   if(copt.count("Macro")) p->setAttribute("Macro", copt["Macro"][0]);
   if(copt.count("GmshOption")) p->setAttribute("GmshOption", copt["GmshOption"][0]);
+  if(copt.count("Units")) p->setAttribute("Units", copt["Units"][0]);
   if(copt.count("AutoCheck")) // for backward compatibility
     p->setAttribute("AutoCheck", copt["AutoCheck"][0]);
 
