@@ -29,13 +29,17 @@ class Msg {
  private:
   // current cpu number and total number of cpus
   static int _commRank, _commSize;
-  // verbosity level (0: silent except fatal errors, 1: +errors, 2:
-  // +warnings, 3: +direct, 4: +info except status bar, 5: normal, 99: debug)
+  // verbosity level (0: silent except fatal errors, 1: +errors, 2: +warnings,
+  // 3: +direct, 4: +info, 5 (=normal): +statusbar, 99: debug)
   static int _verbosity;
   // step (in %) of the progress meter and current progress %
   static int _progressMeterStep, _progressMeterCurrent;
   // timers
   static std::map<std::string, double> _timers;
+  // report cpu time for each info message?
+  static bool _infoCpu;
+  // starting time (gettimeofday at startup)
+  static double _startTime;
   // counters
   static int _warningCount, _errorCount, _atLeastOneErrorInRun;
   static std::string _firstWarning, _firstError;
@@ -80,6 +84,8 @@ class Msg {
   {
     return _commandLineStrings;
   }
+  static std::string PrintResources(bool printDate, bool printWallTime,
+                                    bool printCpu, bool printMem);
   static void Fatal(const char *fmt, ...);
   static void Error(const char *fmt, ...);
   static void Warning(const char *fmt, ...);
@@ -94,6 +100,7 @@ class Msg {
   static void SetProgressMeterStep(int step){ _progressMeterStep = step; }
   static int GetProgressMeterStep(){ return _progressMeterStep; }
   static void ResetProgressMeter(){ if(!_commRank) _progressMeterCurrent = 0; }
+  static void SetInfoCpu(bool val){ _infoCpu = val; }
   static double &Timer(std::string str){ return _timers[str]; }
   static void PrintTimers();
   static void ResetErrorCounter();
@@ -110,6 +117,7 @@ class Msg {
   static void SetExecutableName(const std::string &name) { _execName.assign(name); }
   static std::string GetExecutableName() { return _execName; }
   static void LoadOnelabClient(const std::string &name, const std::string &sockName);
+  static int GetNumOnelabClients();
   static GmshClient *GetGmshClient(){ return _client; }
 #if defined(HAVE_ONELAB)
   static onelab::client *GetOnelabClient(){ return _onelabClient; }
@@ -126,6 +134,9 @@ class Msg {
                                 bool errorIfMissing=false);
   static std::string GetOnelabString(std::string name, const std::string &defaultValue="",
                                      bool errorIfMissing=false);
+  static void AddOnelabStringChoice(std::string name, std::string kind,
+                                    std::string value, bool updateValue=true,
+                                    bool readOnly=false, bool visible=true);
   static void SetOnelabAction(const std::string &action);
   static std::string GetOnelabAction();
   static void ExchangeOnelabParameter(const std::string &key,
