@@ -192,6 +192,9 @@ int ParseFile(const std::string &fileName, bool close, bool warnIfMissing)
     return 0;
   }
 
+  Msg::AddOnelabStringChoice("Gmsh/}Input files", "file", fileName, true, true,
+                             Msg::GetNumOnelabClients() > 1);
+
   //#if defined(HAVE_FLTK) && defined(HAVE_POST)
   // int numViewsBefore = PView::list.size();
   //#endif
@@ -317,6 +320,9 @@ int MergeFile(const std::string &fileName, bool warnIfMissing, bool setBoundingB
 
   Msg::StatusBar(true, "Reading '%s'...", fileName.c_str());
 
+  Msg::AddOnelabStringChoice("Gmsh/}Input files", "file", fileName, true, true,
+                             Msg::GetNumOnelabClients() > 1);
+
   std::vector<std::string> split = SplitFileName(fileName);
   std::string noExt = split[0] + split[1], ext = split[2];
 
@@ -610,12 +616,13 @@ int MergePostProcessingFile(const std::string &fileName, int showViews,
     }
   }
 
-  // if we added steps, go to the last one
+  // if we added steps, and we have more than 2 (to avoid always showing the
+  // imaginary part for complex fields), go to the last one
   if(showLastStep){
     steps.resize(PView::list.size(), 0);
     for(unsigned int i = 0; i < PView::list.size(); i++){
       int step = (int)opt_view_nb_timestep(i, GMSH_GET, 0);
-      if(step > steps[i])
+      if(step > steps[i] && steps[i] > 1)
         opt_view_timestep(i, GMSH_SET|GMSH_GUI, step - 1);
     }
   }
