@@ -109,6 +109,9 @@ class MElement
   // get the vertex using the VTK ordering
   virtual MVertex *getVertexVTK(int num){ return getVertex(num); }
 
+  // get the vertex using the Tochnog ordering
+  virtual MVertex *getVertexTOCHNOG(int num){ return getVertex(num); }
+
   // get the vertex using the Nastran BDF ordering
   virtual MVertex *getVertexBDF(int num){ return getVertex(num); }
 
@@ -380,6 +383,7 @@ class MElement
   virtual void writePLY2(FILE *fp);
   virtual void writeUNV(FILE *fp, int num=0, int elementary=1, int physical=1);
   virtual void writeVTK(FILE *fp, bool binary=false, bool bigEndian=false);
+  virtual void writeTOCHNOG(FILE *fp, int num);
   virtual void writeMESH(FILE *fp, int elementTagType=1, int elementary=1,
                          int physical=0);
   virtual void writeIR3(FILE *fp, int elementTagType, int num, int elementary,
@@ -396,6 +400,7 @@ class MElement
   virtual int getTypeForMSH() const { return 0; }
   virtual int getTypeForUNV() const { return 0; }
   virtual int getTypeForVTK() const { return 0; }
+  virtual const char *getStringForTOCHNOG() const { return 0; }
   virtual const char *getStringForPOS() const { return 0; }
   virtual const char *getStringForBDF() const { return 0; }
   virtual const char *getStringForDIFF() const { return 0; }
