@@ -242,6 +242,7 @@ class MPrism15 : public MPrism {
   virtual int getNumVertices() const { return 15; }
   virtual MVertex *getVertex(int num){ return num < 6 ? _v[num] : _vs[num - 6]; }
   virtual const MVertex *getVertex(int num) const { return num < 6 ? _v[num] : _vs[num - 6]; }
+  virtual void setVertex(int num,  MVertex *v){ if(num < 6) _v[num] = v; else _vs[num - 6] = v; }
   virtual MVertex *getVertexUNV(int num)
   {
     static const int map[15] = {0, 6, 1, 9, 2, 7, 8, 10, 11, 3, 12, 4, 14, 5, 13};
@@ -346,6 +347,7 @@ class MPrism18 : public MPrism {
   virtual int getNumVertices() const { return 18; }
   virtual MVertex *getVertex(int num){ return num < 6 ? _v[num] : _vs[num - 6]; }
   virtual const MVertex *getVertex(int num) const{ return num < 6 ? _v[num] : _vs[num - 6]; }
+  virtual void setVertex(int num,  MVertex *v){ if(num < 6) _v[num] = v; else _vs[num - 6] = v; }
   virtual int getNumEdgeVertices() const { return 9; }
   virtual int getNumFaceVertices() const { return 3; }
   virtual int getNumEdgesRep(bool curved);
@@ -422,7 +424,8 @@ class MPrismN : public MPrism {
   virtual int getPolynomialOrder() const { return _order; }
   virtual int getNumVertices() const { return 6+_vs.size(); }
   virtual MVertex *getVertex(int num){ return num < 6 ? _v[num] : _vs[num-6]; }
-  virtual const MVertex *getVertex(int num) const{ return num < 6 ? _v[num] : _vs[num-6]; }
+  virtual const MVertex *getVertex(int num) const{ return num < 6 ? _v[num] : _vs[num - 6]; }
+  virtual void setVertex(int num,  MVertex *v){ if(num < 6) _v[num] = v; else _vs[num - 6] = v; }
   virtual int getNumEdgeVertices() const { return 9*(_order-1); }
   virtual int getNumFaceVertices() const
   {
