@@ -117,6 +117,7 @@ class MQuadrangle : public MElement {
   virtual const char *getStringForBDF() const { return "CQUAD4"; }
   virtual const char *getStringForDIFF() const { return "ElmB4n2D"; }
   virtual const char *getStringForINP() const { return "CPS4"/*"C2D4"*/; }
+  virtual const char *getStringForTOCHNOG() const { return "-quad4"; }
   virtual void getNode(int num, double &u, double &v, double &w) const
   {
     w = 0.;
@@ -138,10 +139,10 @@ class MQuadrangle : public MElement {
     MVertex *tmp = _v[1]; _v[1] = _v[3]; _v[3] = tmp;
   }
   // reorient the quadrangle to conform with other face
-  // orientation computed with MFace based on this face with respect to other 
+  // orientation computed with MFace based on this face with respect to other
   // in computeCorrespondence
   virtual void reorient(int rotation, bool swap);
-  
+
   virtual bool isInside(double u, double v, double w) const
   {
     double tol = _isInsideTolerance;
@@ -204,6 +205,7 @@ class MQuadrangle8 : public MQuadrangle {
   virtual int getNumVertices() const { return 8; }
   virtual MVertex *getVertex(int num){ return num < 4 ? _v[num] : _vs[num - 4]; }
   virtual const MVertex *getVertex(int num) const { return num < 4 ? _v[num] : _vs[num - 4]; }
+  virtual void setVertex(int num,  MVertex *v){ if(num < 4) _v[num] = v; else _vs[num - 4] = v; }
   virtual MVertex *getVertexUNV(int num)
   {
     static const int map[8] = {0, 4, 1, 5, 2, 6, 3, 7};
@@ -249,10 +251,10 @@ class MQuadrangle8 : public MQuadrangle {
   }
 
   // reorient the quadrangle to conform with other face
-  // orientation computed with MFace based on this face with respect to other 
+  // orientation computed with MFace based on this face with respect to other
   // in computeCorrespondence
   virtual void reorient(int rotation, bool swap);
-  
+
   virtual void getNode(int num, double &u, double &v, double &w) const
   {
     num < 4 ? MQuadrangle::getNode(num, u, v, w) : MElement::getNode(num, u, v, w);
@@ -297,6 +299,7 @@ class MQuadrangle9 : public MQuadrangle {
   virtual int getNumVertices() const { return 9; }
   virtual MVertex *getVertex(int num){ return num < 4 ? _v[num] : _vs[num - 4]; }
   virtual const MVertex *getVertex(int num) const { return num < 4 ? _v[num] : _vs[num - 4]; }
+  virtual void setVertex(int num,  MVertex *v){ if(num < 4) _v[num] = v; else _vs[num - 4] = v; }
   virtual MVertex *getVertexDIFF(int num)
   {
     static const int map[9] = {0, 2, 8, 6, 1, 5, 7, 3, 4};
@@ -328,6 +331,7 @@ class MQuadrangle9 : public MQuadrangle {
   virtual int getTypeForVTK() const { return 28; }
   virtual const char *getStringForPOS() const { return "SQ2"; }
   virtual const char *getStringForDIFF() const { return "ElmB9n2D"; }
+  virtual const char *getStringForTOCHNOG() const { return "-quad9"; }
   virtual void reverse()
   {
     MVertex *tmp;
@@ -337,10 +341,10 @@ class MQuadrangle9 : public MQuadrangle {
   }
 
   // reorient the quadrangle to conform with other face
-  // orientation computed with MFace based on this face with respect to other 
+  // orientation computed with MFace based on this face with respect to other
   // in computeCorrespondence
   virtual void reorient(int rotation, bool swap);
-  
+
   virtual void getNode(int num, double &u, double &v, double &w) const
   {
     num < 4 ? MQuadrangle::getNode(num, u, v, w) : MElement::getNode(num, u, v, w);
@@ -459,12 +463,12 @@ class MQuadrangleN : public MQuadrangle {
     inv.insert(inv.begin(), _vs.rbegin(), _vs.rend());
     _vs = inv;
   }
-  
+
   // reorient the quadrangle to conform with other face
-  // orientation computed with MFace based on this face with respect to other 
+  // orientation computed with MFace based on this face with respect to other
   // in computeCorrespondence
   virtual void reorient(int rotation, bool swap);
-  
+
   virtual void getNode(int num, double &u, double &v, double &w) const
   {
     num < 4 ? MQuadrangle::getNode(num, u, v, w) : MElement::getNode(num, u, v, w);
