@@ -120,16 +120,17 @@ class MTriangle : public MElement {
   virtual const char *getStringForBDF() const { return "CTRIA3"; }
   virtual const char *getStringForDIFF() const { return "ElmT3n2D"; }
   virtual const char *getStringForINP() const { return "CPS3"/*"STRI3"*//*"C2D3"*/; }
+  virtual const char *getStringForTOCHNOG() const { return "-tria3"; }
   virtual void reverse()
   {
     MVertex *tmp = _v[1]; _v[1] = _v[2]; _v[2] = tmp;
   }
-  
+
   // reorient the triangle to conform with other face
-  // orientation computed with MFace based on this face with respect to other 
+  // orientation computed with MFace based on this face with respect to other
   // in computeCorrespondence
   virtual void reorient(int rotation, bool swap);
-  
+
   virtual void getNode(int num, double &u, double &v, double &w) const
   {
     w = 0.;
@@ -199,6 +200,7 @@ class MTriangle6 : public MTriangle {
   virtual int getNumVertices() const { return 6; }
   virtual MVertex *getVertex(int num){ return num < 3 ? _v[num] : _vs[num - 3]; }
   virtual const MVertex *getVertex(int num) const { return num < 3 ? _v[num] : _vs[num - 3]; }
+  virtual void setVertex(int num,  MVertex *v){ if(num < 3) _v[num] = v; else _vs[num - 3] = v; }
   virtual MVertex *getVertexUNV(int num)
   {
     static const int map[6] = {0, 3, 1, 4, 2, 5};
@@ -231,6 +233,7 @@ class MTriangle6 : public MTriangle {
   virtual const char *getStringForBDF() const { return "CTRIA6"; }
   virtual const char *getStringForDIFF() const { return "ElmT6n2D"; }
   virtual const char *getStringForINP() const { return "CPS6"/*"STRI65"*//*"C2D6"*/; }
+  virtual const char *getStringForTOCHNOG() const { return "-tria6"; }
   virtual void reverse()
   {
     MVertex *tmp;
@@ -242,7 +245,7 @@ class MTriangle6 : public MTriangle {
     num < 3 ? MTriangle::getNode(num, u, v, w) : MElement::getNode(num, u, v, w);
   }
   // reorient the triangle to conform with other face
-  // orientation computed with MFace based on this face with respect to other 
+  // orientation computed with MFace based on this face with respect to other
   // in computeCorrespondence
   virtual void reorient(int rotation, bool swap);
 };
@@ -285,6 +288,7 @@ class MTriangleN : public MTriangle {
   virtual int getNumVertices() const { return 3 + _vs.size(); }
   virtual MVertex *getVertex(int num){ return num < 3 ? _v[num] : _vs[num - 3]; }
   virtual const MVertex *getVertex(int num) const { return num < 3 ? _v[num] : _vs[num - 3]; }
+  virtual void setVertex(int num,  MVertex *v){ if(num < 3) _v[num] = v; else _vs[num - 3] = v; }
   virtual int getNumFaceVertices() const
   {
     if (getIsAssimilatedSerendipity())
@@ -353,7 +357,7 @@ class MTriangleN : public MTriangle {
     num < 3 ? MTriangle::getNode(num, u, v, w) : MElement::getNode(num, u, v, w);
   }
   // reorient the triangle to conform with other face
-  // orientation computed with MFace based on this face with respect to other 
+  // orientation computed with MFace based on this face with respect to other
   // in computeCorrespondence
   virtual void reorient(int rotation, bool swap);
 };
