@@ -3,7 +3,7 @@
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
 
-#include <queue> // #mark
+#include <queue> 
 #include <stdlib.h>
 #include "GmshConfig.h"
 
@@ -19,13 +19,18 @@
 #include "Context.h"
 #include "OS.h"
 #include "ANN/ANN.h"
-#include "linearSystemCSR.h"
-#include "dofManager.h"
 #include "laplaceTerm.h"
 #include "convexLaplaceTerm.h"
-#include "convexCombinationTerm.h"  // #FIXME
+#include "convexCombinationTerm.h"
 
-#include "qualityMeasuresJacobian.h" // #temporary?
+#if defined(HAVE_MESH)
+#include "qualityMeasuresJacobian.h"
+#endif
+
+#if defined(HAVE_OPTHOM)
+#include "OptHomRun.h"
+#include "MeshQualityOptimizer.h"
+#endif
 
 static inline void functionShapes(int p, double Xi[2], double* phi)
 {
@@ -203,7 +208,7 @@ static bool orderVertices(const double &tot_length, const std::vector<MVertex*>
 				(next->z() - first->z()) * (next->z() - first->z()) );
 
     coord.push_back(coord[coord.size()-1] + length / tot_length);
-    
+
     first = next;
 
   }
@@ -213,12 +218,12 @@ static bool orderVertices(const double &tot_length, const std::vector<MVertex*>
 /*BUILDER*/
 discreteDiskFace::discreteDiskFace(GFace *gf, triangulation* diskTriangulation,
                                    int p, std::vector<GFace*> *CAD) :
-  GFace(gf->model(),123), _parent (gf), _ddft(NULL), oct(NULL)
+  GFace(gf->model(),diskTriangulation->idNum), _parent (gf), _ddft(NULL), oct(NULL)
 {
   initialTriangulation = diskTriangulation;
   std::vector<MElement*> mesh = diskTriangulation->tri;
   _order = p;
-  _N = (p+1)*(p+2)/2;
+  _n = (p+1)*(p+2)/2;
   discrete_triangles.resize(mesh.size());
   std::map<MVertex*,MVertex*> v2v;// mesh vertex |-> face vertex
   std::map<MEdge,MVertex*,Less_Edge> ed2nodes; // edge to interior node(s)
@@ -251,7 +256,7 @@ discreteDiskFace::discreteDiskFace(GFace *gf, triangulation* diskTriangulation,
 	}
 	else vs.push_back(v);
       }
-      else vs.push_back(v);          
+      else vs.push_back(v);
     }
     if (_order == 2) {// then, interior nodes :-)
       for (unsigned int ie=0; ie<3; ie++){ // firstly, edge interior nodes :-)
@@ -264,33 +269,36 @@ discreteDiskFace::discreteDiskFace(GFace *gf, triangulation* diskTriangulation,
 	  discrete_vertices.push_back(mv);
 	  ed2nodes[me]=mv;
 	}
-	else vs.push_back(it->second);	
+	else vs.push_back(it->second);
       }
     }// end order == 2
     if (_order==1)
       discrete_triangles[i] = new MTriangle (vs);
     else if (_order==2)
       discrete_triangles[i] = new MTriangle6 (vs);
-        
+
   }// end loop over triangles
-  geoTriangulation = new triangulation(discrete_triangles,gf);
+  geoTriangulation = new triangulation(tag(),discrete_triangles,gf);
+  geoTriangulation->fillingHoles = diskTriangulation->fillingHoles;
   allNodes = geoTriangulation->vert;
   _totLength = geoTriangulation->bord.rbegin()->first;
   _U0 = geoTriangulation->bord.rbegin()->second;
   orderVertices(_totLength, _U0, _coords);
   parametrize(false);
   buildOct(CAD);
-  
-  if (!checkOrientationUV()){
-    Msg::Info("discreteDiskFace:: parametrization is not one-to-one; fixing "
-              "the discrete system.");
-    parametrize(true);
-    buildOct(CAD);    
-  }
-  
-  putOnView(true,false);
   printParamMesh();
-  if(!checkOrientationUV()) Msg::Fatal("discreteDiskFace:: failing to fix the discrete system");
+  if (!checkOrientationUV()){
+    Msg::Info("discreteDiskFace:: parametrization is not one-to-one; it is going to be corrected.");
+    if(_order==1)
+      parametrize(true);
+    else
+      optimize();
+    buildOct(CAD);
+    printParamMesh();
+  }
+  putOnView(true,false);
+  if(!checkOrientationUV()) Msg::Fatal("discreteDiskFace:: failing to compute a one-to-one mapping");
+  else Msg::Info("Parameterization done :-)");
 }
 /*end BUILDER*/
 
@@ -311,36 +319,32 @@ void discreteDiskFace::buildOct(std::vector<GFace*> *CAD) const
   const int maxElePerBucket = 15;
   oct = Octree_Create(maxElePerBucket, origin, ssize, discreteDiskFaceBB,
 		      discreteDiskFaceCentroid, discreteDiskFaceInEle);
-
-  _ddft = new discreteDiskFaceTriangle[discrete_triangles.size()];
-
-  //  if (CAD) printf("------------->  %ld %ld\n",CAD->size(),discrete_triangles.size());
-
+  
+  _ddft = new discreteDiskFaceTriangle[discrete_triangles.size()/*-geoTriangulation->fillingHoles.size()*/];
+  int c = 0;
   for(unsigned int i = 0; i < discrete_triangles.size(); ++i){
+    //    if(geoTriangulation->fillingHoles.find(i)==geoTriangulation->fillingHoles.end()){
     MElement *t = discrete_triangles[i];
-    discreteDiskFaceTriangle* my_ddft = &_ddft[i];
-    my_ddft->p.resize(_N);
-    for(int io=0; io<_N; io++)
+    discreteDiskFaceTriangle* my_ddft = &_ddft[c++];
+    my_ddft->p.resize(_n);
+    for(int io=0; io<_n; io++)
       my_ddft->p[io] = coordinates[t->getVertex(io)];
 
     my_ddft->gf = CAD ? (*CAD)[i] : _parent;
     my_ddft->tri = t;
 
     Octree_Insert(my_ddft, oct);
+    //    }
   }
   Octree_Arrange(oct);
 }
 
-bool discreteDiskFace::parametrize(bool one2one) const
+bool discreteDiskFace::parametrize(bool one2one)// const
 { // #improveme
 
-  if (one2one)
-    Msg::Info("Parametrizing surface %d with 'one-to-one map'", tag());
-  else
-    Msg::Info("Parametrizing surface %d with 'harmonic map'", tag());
 
-  linearSystem<double> *lsys_u;
-  linearSystem<double> *lsys_v;
+  linearSystemCSRTaucs<double> *lsys_u;
+  linearSystemCSRTaucs<double> *lsys_v;
 
   lsys_u = new linearSystemCSRTaucs<double>; // taucs :-)
   lsys_v = new linearSystemCSRTaucs<double>; // taucs :-)
@@ -348,16 +352,18 @@ bool discreteDiskFace::parametrize(bool one2one) const
   dofManager<double> myAssemblerU(lsys_u);   // hashing
   dofManager<double> myAssemblerV(lsys_v);
 
-  for(size_t i = 0; i < _U0.size(); i++){
-    MVertex *v = _U0[i];
-    const double theta = 2 * M_PI * _coords[i];
-    if(i%_order==0){
-      myAssemblerU.fixVertex(v, 0, 1,cos(theta));
-      myAssemblerV.fixVertex(v, 0, 1,sin(theta));
-    }
-    else{//#TEST
-      myAssemblerU.fixVertex(v, 0, 1,1./_order*((_order-(i%_order))*cos(2*M_PI*_coords[i-(i%_order)])+(i%_order)*cos(2*M_PI*_coords[i+(_order-(i%_order))])));
-      myAssemblerV.fixVertex(v, 0, 1,1./_order*((_order-(i%_order))*sin(2*M_PI*_coords[i-(i%_order)])+(i%_order)*sin(2*M_PI*_coords[i+(_order-(i%_order))])));
+  if(!one2one){
+    for(size_t i = 0; i < _U0.size(); i++){
+      MVertex *v = _U0[i];
+      const double theta = 2 * M_PI * _coords[i];
+      if(i%_order==0){
+	myAssemblerU.fixVertex(v, 0, 1,cos(theta));
+	myAssemblerV.fixVertex(v, 0, 1,sin(theta));
+      }
+      else{//#TEST
+	myAssemblerU.fixVertex(v, 0, 1,1./_order*((_order-(i%_order))*cos(2*M_PI*_coords[i-(i%_order)])+(i%_order)*cos(2*M_PI*_coords[i+(_order-(i%_order))])));
+	myAssemblerV.fixVertex(v, 0, 1,1./_order*((_order-(i%_order))*sin(2*M_PI*_coords[i-(i%_order)])+(i%_order)*sin(2*M_PI*_coords[i+(_order-(i%_order))])));
+      }
     }
   }
 
@@ -376,28 +382,22 @@ bool discreteDiskFace::parametrize(bool one2one) const
   double t1 = Cpu();
 
   simpleFunction<double> ONE(1.0);
+  laplaceTerm mappingU(0, 1, &ONE);
+  laplaceTerm mappingV(0, 1, &ONE);
 
-  if (one2one){
-    convexLaplaceTerm mappingU(0, 1, &ONE);
-    convexLaplaceTerm mappingV(0, 1, &ONE);
-
-    for(unsigned int i = 0; i < discrete_triangles.size(); ++i){
-      SElement se(discrete_triangles[i]);
-      mappingU.addToMatrix(myAssemblerU, &se);
-      mappingV.addToMatrix(myAssemblerV, &se);
-    }
-  }
-  else{
-    laplaceTerm mappingU(0, 1, &ONE);
-    laplaceTerm mappingV(0, 1, &ONE);
-
-    for(unsigned int i = 0; i < discrete_triangles.size(); ++i){
-      SElement se(discrete_triangles[i]);
-      mappingU.addToMatrix(myAssemblerU, &se);
-      mappingV.addToMatrix(myAssemblerV, &se);
-    }
+  for(unsigned int i = 0; i < discrete_triangles.size(); ++i){
+    SElement se(discrete_triangles[i]);
+    mappingU.addToMatrix(myAssemblerU, &se);
+    mappingV.addToMatrix(myAssemblerV, &se);
   }
 
+
+  if(one2one){
+    Msg::Info("discreteDiskFace::parametrize \t Modifying discrete system");
+    checklsys(lsys_u,&myAssemblerU,1);
+    checklsys(lsys_v,&myAssemblerV,0);
+  }
+     
   double t2 = Cpu();
   Msg::Debug("Assembly done in %8.3f seconds", t2 - t1);
   lsys_u->systemSolve();
@@ -420,13 +420,100 @@ bool discreteDiskFace::parametrize(bool one2one) const
       itf->second[1]= value_V;
     }
   }
+  Msg::Debug("Systems saved");
 
   delete lsys_u;
   delete lsys_v;
 
+  Msg::Debug("Systems deleted");
   return true;
 }
 
+void discreteDiskFace::checklsys(linearSystemCSRTaucs<double>* lsys,dofManager<double>* myAssembler,int U)
+{
+     
+  INDEX_TYPE *ai;
+  INDEX_TYPE *jptr;
+  double *a;
+  //double b;
+  lsys->getMatrix(jptr,ai,a);
+  int NbUnk = lsys->getNbUnk();
+  //  int NNZ = lsys->getNNZ();
+  double tr = 0.;
+  //printf("------------------------%d\n######################%d\n______________________\n",NNZ,NbUnk);
+  for(int i=0; i<NbUnk; i++){
+    tr += a[jptr[i]];
+    /*lsys->getFromRightHandSide(i,b);
+    printf("%d: %d->%d\n",i,jptr[i],jptr[i+1]);
+    for(int j=jptr[i]; j<jptr[i+1]; j++)
+      printf("%d: %f \t",ai[j],a[j]);
+      printf("=%f \n",b);*/
+  }
+  tr /= NbUnk;
+
+  double alpha = .1;
+  for(int i=0; i<NbUnk; i++)
+    for(int j=jptr[i]+1; j<jptr[i+1]; j++)
+      if(a[j] > -alpha*tr){
+	double delta = a[j] + alpha*tr;
+	a[jptr[i]] += delta;
+	a[jptr[ai[j]]] += delta;
+	a[j] -= delta;
+      }
+  /*
+  printf("\n");
+  for(int i=0; i<NbUnk; i++){
+    lsys->getFromRightHandSide(i,b);
+    printf("%d: %d->%d\n",i,jptr[i],jptr[i+1]);
+    for(int j=jptr[i]; j<jptr[i+1]; j++)
+      printf("%d: %f \t",ai[j],a[j]);
+    printf("=%f \n",b);
+  }
+  */
+  std::map<int,int> dirichlet;
+  for(unsigned int i=0; i<_U0.size(); i++)
+    dirichlet[myAssembler->getDofNumber(Dof(_U0[i]->getNum(),10000))] = i;
+
+
+  for(int i=0; i<NbUnk; i++){
+    if(dirichlet.find(i)!=dirichlet.end()){
+      lsys->addToMatrix(i,i,-a[jptr[i]]+1);
+      double theta = 2 * M_PI *  _coords[dirichlet[i]];
+      double val = U*cos(theta) + (1-U)*sin(theta);
+      lsys->addToRightHandSide(i,val);
+      for(int j=jptr[i]+1; j<jptr[i+1]; j++){
+	if(dirichlet.find(ai[j])==dirichlet.end())
+	   lsys->addToRightHandSide(ai[j],-a[j]*val);
+	a[j] = 0;
+      }
+    }
+    else{
+      for(int j=jptr[i]+1; j<jptr[i+1]; j++){
+	if(dirichlet.find(ai[j])!=dirichlet.end()){
+	  double theta = 2 * M_PI *  _coords[dirichlet[ai[j]]];
+	  double val = U*cos(theta) + (1-U)*sin(theta);
+	  lsys->addToRightHandSide(i,-a[j]*val);
+	  a[j] = 0;
+	}
+      }
+    }//end else
+  }//end for i
+  /*
+  lsys->getMatrix(jptr,ai,a);
+  NbUnk = lsys->getNbUnk();
+  NNZ = lsys->getNNZ();
+  printf("--#--#--#--\n");
+  for(int i=0; i<NbUnk; i++){
+    lsys->getFromRightHandSide(i,b);
+    printf("%d: %d->%d\n",i,jptr[i],jptr[i+1]);
+    for(int j=jptr[i]; j<jptr[i+1]; j++)
+      printf("%d: %f \t",ai[j],a[j]);
+    printf("=%f \n",b);
+  }
+ */
+  
+}
+
 void discreteDiskFace::getTriangleUV(const double u,const double v,
 				     discreteDiskFaceTriangle **mt,
 				     double &_xi, double &_eta)const{
@@ -450,48 +537,161 @@ bool discreteDiskFace::checkOrientationUV()
   discreteDiskFaceTriangle *ct;
 
   if(_order==1){
-    double initial, current; // initial and current orientation
+    double current; // initial and current orientation
     ct = &_ddft[0];
     double p1[2] = {ct->p[0].x(), ct->p[0].y()};
     double p2[2] = {ct->p[1].x(), ct->p[1].y()};
     double p3[2] = {ct->p[2].x(), ct->p[2].y()};
-    initial = robustPredicates::orient2d(p1, p2, p3);
-    unsigned int i=1;
-    for (; i<discrete_triangles.size(); i++){
+    unsigned int nbP = 0;
+    unsigned int nbM = 0;
+    for (unsigned int i=0; i<discrete_triangles.size(); i++){
       ct = &_ddft[i];
       p1[0] = ct->p[0].x(); p1[1] = ct->p[0].y();
       p2[0] = ct->p[1].x(); p2[1] = ct->p[1].y();
       p3[0] = ct->p[2].x(); p3[1] = ct->p[2].y();
       current = robustPredicates::orient2d(p1, p2, p3);
-      if(initial*current < 0.) {
-	Msg::Error("Triangle UV %d has not the correct orientation",i+1);
-	return false;
-	break;
-      }
+      if(current < 0.) nbM++;
+      else nbP++;
+    }
+    if (nbP*nbM){
+      Msg::Info("Map %d of the atlas : Triangles have different orientations (%d + / %d -)",tag(),nbP,nbM);
+      return false;
     }
     return true;
   }
   else{
+#if defined(HAVE_MESH)
     double min, max;
     std::vector<MVertex*> localVertices;
-    localVertices.resize(_N);
-    for(unsigned int i=0; i<discrete_triangles.size(); i++){
+    localVertices.resize(_n);
+    for(unsigned int i=0; i<discrete_triangles.size()-geoTriangulation->fillingHoles.size(); i++){
       ct = &_ddft[i];
-      for(int j=0; j<_N; j++)
+      for(int j=0; j<_n; j++)
 	localVertices[j] = new MVertex(ct->p[j].x(),ct->p[j].y(),0.);
       MTriangle6 mt6(localVertices);
       jacobianBasedQuality::minMaxJacobianDeterminant(&mt6,min,max);
-      for(int j=0; j<_N; j++)
+      for(int j=0; j<_n; j++)
 	delete localVertices[j];
       if (min < 0){
 	return false;
 	break;
       }
     }
+#endif
     return true;
   }
 }
 
+void discreteDiskFace::optimize()
+{ // #improve . . .
+#if defined(HAVE_OPTHOM)
+
+  // parameters for mesh optimization
+  // -- high order
+  OptHomParameters optParams;
+  optParams.dim = 2;
+  optParams.optPassMax = 100;
+  optParams.TMAX = 300;
+  optParams.fixBndNodes = true;
+  optParams.optPrimSurfMesh = true;
+  optParams.BARRIER_MIN = 1e-3;
+  optParams.BARRIER_MAX = 1e3;
+  optParams.strategy = 0;
+  // -- linear
+  MeshQualOptParameters opt;
+  opt.dim = 2;
+  opt.fixBndNodes = true;
+
+  //creating the "geometry" of the parametrization, and its corresponding mesh
+  // -- generation of parametric nodes
+  std::map<SPoint3,MVertex*> sp2mv;
+  std::vector<MElement*> paramTriangles;
+  for(std::map<MVertex*,SPoint3>::iterator it=coordinates.begin(); it!= coordinates.end(); ++it)
+    sp2mv[it->second] = new MVertex(it->second.x(),it->second.y(),0.);
+  // -- generation of parametric triangles
+  paramTriangles.resize(discrete_triangles.size() - geoTriangulation->fillingHoles.size());
+  for(unsigned int i=0; i<discrete_triangles.size() -geoTriangulation->fillingHoles.size(); i++){
+    discreteDiskFaceTriangle* ct = &_ddft[i];
+    std::vector<MVertex*> mv;
+    mv.resize(ct->tri->getNumVertices());
+    for (int j=0; j<ct->tri->getNumVertices(); j++)
+      mv[j] = sp2mv[ct->p[j]];
+    if(_order==1)
+      paramTriangles[i] = new MTriangle(mv);
+    else
+      paramTriangles[i] = new MTriangle6(mv);
+  }
+  // -- generation of the parametric topology #mark what about the GFace for the GModel ?
+  std::map<int,std::vector<MElement*> > e2e;// entity to element
+  e2e[0] = paramTriangles;
+  std::vector<int> v;
+  v.push_back(0);
+  std::map<int,std::vector<int> > e2p;// entity to physical
+  e2p[0] = v;
+  std::set<MVertex*> todelete;
+  GModel* paramDisk = GModel::createGModel(e2e,e2p);
+  // ---- discrete vertex
+  std::set<GFace*, GEntityLessThan>::iterator it = paramDisk->firstFace();
+  GFace *dgf = *it;
+  discreteVertex *dv = new discreteVertex(paramDisk,paramDisk->getMaxElementaryNumber(0)+1);
+  sp2mv[coordinates[_U0[0]]]->setEntity(dv);
+  dv->mesh_vertices.push_back(sp2mv[coordinates[_U0[0]]]);
+  todelete.insert(sp2mv[coordinates[_U0[0]]]);
+  paramDisk->add(dv);
+  // ---- discrete edge
+  discreteEdge *de = new discreteEdge(paramDisk,paramDisk->getMaxElementaryNumber(1)+1,dv,dv);
+  paramDisk->add(de);
+  std::vector<MLine*> lines;
+  for(unsigned int i=1; i<_U0.size(); i++){
+    sp2mv[coordinates[_U0[i]]]->setEntity(de);
+    de->mesh_vertices.push_back(sp2mv[coordinates[_U0[i]]]);
+    todelete.insert(sp2mv[coordinates[_U0[i]]]);
+    lines.push_back(new MLine(sp2mv[coordinates[_U0[i-1]]],sp2mv[coordinates[_U0[i]]]));
+  }
+  lines.push_back(new MLine(sp2mv[coordinates[_U0[_U0.size()-1]]],sp2mv[coordinates[_U0[0]]]));
+  de->setTopo(lines);
+  de->createGeometry();// !!!! setTopo ... MLine's
+ 
+  
+  
+  // optimization
+  if(_order >1)
+    HighOrderMeshOptimizer(paramDisk, optParams);
+  else
+    MeshQualityOptimizer(paramDisk,opt);
+
+ 
+  
+  
+  // update the parametrization
+  paramTriangles = e2e[0];
+  for(unsigned int i=0; i< paramTriangles.size(); i++){
+    discreteDiskFaceTriangle* ct = &_ddft[i];
+    MElement* tri = paramTriangles[i];
+    for(int j=0; j<ct->tri->getNumVertices(); j++){
+      MVertex* mv = tri->getVertex(j);
+      SPoint3 p(mv->x(), mv->y(), 0);
+      coordinates[ct->tri->getVertex(j)] = p;
+      ct->p[j] = p;
+    }
+  }
+
+  // update GFace's mesh_vertices
+  std::vector<MVertex*> newMV;
+  for(unsigned int imv=0; imv<dgf->mesh_vertices.size(); imv++){
+    MVertex* current = dgf->mesh_vertices[imv];
+    std::set<MVertex*>::iterator itmv = todelete.find(current);
+    if (itmv==todelete.end()) newMV.push_back(current);
+  }
+  dgf->mesh_vertices.clear();
+  dgf->mesh_vertices = newMV;
+  
+  // cleaning
+  delete paramDisk;
+  
+#endif
+}
+
 // (u;v) |-> < (x;y;z); GFace; (u;v) >
 GPoint discreteDiskFace::point(const double par1, const double par2) const
 {
@@ -506,10 +706,10 @@ GPoint discreteDiskFace::point(const double par1, const double par2) const
     return gp;
   }
 
-  std::vector<double> eval(_N);
+  std::vector<double> eval(_n);
   functionShapes(_order,Xi,&eval[0]);
   double X=0,Y=0,Z=0;
-  for(int io=0; io<_N; io++){
+  for(int io=0; io<_n; io++){
     X += dt->tri->getVertex(io)->x()*eval[io];
     Y += dt->tri->getVertex(io)->y()*eval[io];
     Z += dt->tri->getVertex(io)->z()*eval[io];
@@ -591,7 +791,7 @@ Pair<SVector3, SVector3> discreteDiskFace::firstDer(const SPoint2 &param) const
 
   double dudxi[2][2] = {{0.,0.},{0.,0.}};
 
-  for (int io=0; io<_N; io++){
+  for (int io=0; io<_n; io++){
 
     double X = tri->getVertex(io)->x();
     double Y = tri->getVertex(io)->y();
@@ -642,16 +842,16 @@ void discreteDiskFace::secondDer(const SPoint2 &param,
 }
 
 void discreteDiskFace::putOnView(bool Xu, bool Ux)
-{
+{// #improveme  using built-in methods
 
   char mybuffer [64];
-  
+
   FILE *view_u=NULL, *view_v=NULL, *UVx=NULL, *UVy=NULL, *UVz=NULL;
 
-  
+
 
   if(Xu){
-   
+
     sprintf(mybuffer, "param_u_part%d_order%d.pos",
 	    initialTriangulation->idNum,_order);
     view_u = Fopen(mybuffer,"w");
@@ -664,7 +864,7 @@ void discreteDiskFace::putOnView(bool Xu, bool Ux)
 
     sprintf(mybuffer, "UVx_part%d_order%d.pos",
 	    initialTriangulation->idNum,_order);
-  
+
     UVx = Fopen(mybuffer,"w");
 
     sprintf(mybuffer, "UVy_part%d_order%d.pos",
@@ -675,7 +875,7 @@ void discreteDiskFace::putOnView(bool Xu, bool Ux)
 	    initialTriangulation->idNum,_order);
     UVz = Fopen(mybuffer,"w");
   }
-  
+
   if((Xu && view_u && view_v) || (Ux && UVx && UVy && UVz)){
     if(Xu){
       fprintf(view_u,"View \"u(X)\"{\n");
@@ -686,7 +886,7 @@ void discreteDiskFace::putOnView(bool Xu, bool Ux)
       fprintf(UVy,"View \"y(U)\"{\n");
       fprintf(UVz,"View \"z(U)\"{\n");
     }
-    for (unsigned int i=0; i<discrete_triangles.size(); i++){
+    for (unsigned int i=0; i<discrete_triangles.size()-geoTriangulation->fillingHoles.size(); i++){
       discreteDiskFaceTriangle* my_ddft = &_ddft[i];
       if (_order == 1){
 	if(Xu){
@@ -710,7 +910,7 @@ void discreteDiskFace::putOnView(bool Xu, bool Ux)
 	  fprintf(UVz,"ST%d(",_order);
 	}
       }
-      for (int j=0; j<_N-1; j++){
+      for (int j=0; j<_n-1; j++){
 	if(Xu){
 	  fprintf(view_u,"%g,%g,%g,",my_ddft->tri->getVertex(j)->x(),
 		  my_ddft->tri->getVertex(j)->y(),my_ddft->tri->getVertex(j)->z());
@@ -724,17 +924,17 @@ void discreteDiskFace::putOnView(bool Xu, bool Ux)
 	}
       }
       if(Xu){
-	fprintf(view_u,"%g,%g,%g){",my_ddft->tri->getVertex(_N-1)->x(),
-		my_ddft->tri->getVertex(_N-1)->y(),my_ddft->tri->getVertex(_N-1)->z());
-	fprintf(view_v,"%g,%g,%g){",my_ddft->tri->getVertex(_N-1)->x(),
-		my_ddft->tri->getVertex(_N-1)->y(),my_ddft->tri->getVertex(_N-1)->z());
+	fprintf(view_u,"%g,%g,%g){",my_ddft->tri->getVertex(_n-1)->x(),
+		my_ddft->tri->getVertex(_n-1)->y(),my_ddft->tri->getVertex(_n-1)->z());
+	fprintf(view_v,"%g,%g,%g){",my_ddft->tri->getVertex(_n-1)->x(),
+		my_ddft->tri->getVertex(_n-1)->y(),my_ddft->tri->getVertex(_n-1)->z());
       }
       if(Ux){
-	fprintf(UVx,"%g,%g,%g){",my_ddft->p[_N-1].x(),my_ddft->p[_N-1].y(),0.);
-	fprintf(UVy,"%g,%g,%g){",my_ddft->p[_N-1].x(),my_ddft->p[_N-1].y(),0.);
-	fprintf(UVz,"%g,%g,%g){",my_ddft->p[_N-1].x(),my_ddft->p[_N-1].y(),0.);
+	fprintf(UVx,"%g,%g,%g){",my_ddft->p[_n-1].x(),my_ddft->p[_n-1].y(),0.);
+	fprintf(UVy,"%g,%g,%g){",my_ddft->p[_n-1].x(),my_ddft->p[_n-1].y(),0.);
+	fprintf(UVz,"%g,%g,%g){",my_ddft->p[_n-1].x(),my_ddft->p[_n-1].y(),0.);
       }
-      for (int j=0; j<_N-1; j++){
+      for (int j=0; j<_n-1; j++){
 	if(Xu){
 	  fprintf(view_u,"%g,",my_ddft->p[j].x());
 	  fprintf(view_v,"%g,",my_ddft->p[j].y());
@@ -746,13 +946,13 @@ void discreteDiskFace::putOnView(bool Xu, bool Ux)
 	}
       }
       if(Xu){
-	fprintf(view_u,"%g};\n",my_ddft->p[_N-1].x());
-	fprintf(view_v,"%g};\n",my_ddft->p[_N-1].y());
+	fprintf(view_u,"%g};\n",my_ddft->p[_n-1].x());
+	fprintf(view_v,"%g};\n",my_ddft->p[_n-1].y());
       }
       if(Ux){
-	fprintf(UVx,"%g};\n",my_ddft->tri->getVertex(_N-1)->x());
-	fprintf(UVy,"%g};\n",my_ddft->tri->getVertex(_N-1)->y());
-	fprintf(UVz,"%g};\n",my_ddft->tri->getVertex(_N-1)->z());
+	fprintf(UVx,"%g};\n",my_ddft->tri->getVertex(_n-1)->x());
+	fprintf(UVy,"%g};\n",my_ddft->tri->getVertex(_n-1)->y());
+	fprintf(UVz,"%g};\n",my_ddft->tri->getVertex(_n-1)->z());
       }
     }
     if(Xu){
@@ -784,7 +984,7 @@ GPoint discreteDiskFace::intersectionWithCircle(const SVector3 &n1, const SVecto
 {
   SVector3 n = crossprod(n1,n2);
   n.normalize();
-  for (int i=-1;i<(int)discrete_triangles.size();i++){
+  for (int i=-1;i<(int)(discrete_triangles.size()-geoTriangulation->fillingHoles.size());i++){
     discreteDiskFaceTriangle *ct = NULL;
     double U,V;
     if (i == -1) getTriangleUV(uv[0],uv[1], &ct, U,V);
@@ -883,12 +1083,238 @@ GPoint discreteDiskFace::intersectionWithCircle(const SVector3 &n1, const SVecto
   return pp;
 }
 
+// computes some kind of maximal distance in a mesh
+
+static void addTo (std::map<MVertex*, std::vector<MElement*> > &v2t, MVertex *v, MElement *t)
+{
+  std::map<MVertex*, std::vector<MElement*> > :: iterator it = v2t.find(v);
+  if (it == v2t.end()){
+    std::vector<MElement*> tt; tt.push_back(t);
+    v2t[v] = tt;
+  }
+  else it->second.push_back(t);
+}
+
+static void update(std::map<MVertex*,double> &Close, MVertex *v2, double d)
+{
+  std::map<MVertex*,double>::iterator it = Close.find(v2);
+  if (it == Close.end())Close[v2] = d;
+  else if (it->second > d) it->second=d;
+  //  printf("DISTANCE COMPUTED %lf\n",d);
+
+}
+
+static MEdge getEdge (MElement *t, MVertex *v)
+{
+  for (int i=0;i<3;i++)
+    if (t->getVertex(i) == v) return t->getEdge((i+1)%3);
+  return MEdge();
+}
+
+/* #warning
+static double computeDistanceLinePoint (MVertex *v1, MVertex *v2, MVertex *v){
+
+  SVector3 U  = v2->point() - v1->point();
+  SVector3 BA = v2->point() - v->point();
+
+  SVector3 xx = crossprod(U,BA);
+  return xx.norm() / U.norm();
+
+}
+*/
+inline double computeDistance (MVertex *v1, double d1, MVertex *v2, double d2, MVertex *v){
+  
+  //       o------------a
+  //
+  //
+  //    x
+  //
+
+  // x^2 + y^2 = d_1^2
+  // (x-a)^2 + y^2 = d_2^2
+  // 2ax - a^2 = d_1^2 - d_2^2
+
+  //  printf("%p %p %p\n",v1,v2,v);
+
+  return std::min(d2+v2->distance(v),d1+v1->distance(v));
+
+
+
+
+  double a = v2->distance(v1);
+
+  // center (seed) to compute the distance (put it down)
+  double x0 = 0.5*(d1*d1-d2*d2+a*a)/a;
+  double y0 = -sqrt ( d1*d1 - x0*x0);
+  //  printf("a %g x0 %g %g d %g %g\n",a,x0,y0,d1,d2);
+
+  // compute coordinates of v in the same system
+  d1 = v1->distance(v);
+  d2 = v2->distance(v);
+  double xv = 0.5*(d1*d1-d2*d2+a*a)/a;
+  double yv = +sqrt ( d1*d1 - x0*x0);
+  return sqrt ((x0-xv)*(x0-xv)+(y0-yv)*(y0-yv));
+}
+
+std::map<MVertex*,double>::iterator closest (std::map<MVertex*,double> &Close)
+{
+  std::map<MVertex*,double>::iterator itClose;
+  double c = 1.e22;
+  for (std::map<MVertex*,double>::iterator it = Close.begin(); it != Close.end(); ++it){
+    if (it->second < c){
+      c = it->second;
+      itClose = it;
+    }
+  }
+  return itClose;
+}
+
+double triangulation::geodesicDistance ()
+{
+  if (bord.empty())return 1.e22;
+  std::map<MVertex*, std::vector<MElement*> > v2t;
+  for (size_t i=0;i<tri.size();++i){
+    addTo (v2t, tri[i]->getVertex(0),tri[i]);
+    addTo (v2t, tri[i]->getVertex(1),tri[i]);
+    addTo (v2t, tri[i]->getVertex(2),tri[i]);
+  }
+
+  //  printf("computing geodesic distance with %d triangles and %d vertices\n",
+  //	 tri.size(),v2t.size());
+
+  std::map<MVertex*,double> Fixed;
+  std::map<MVertex*,double> Close;
+
+  unsigned int N = bord.rbegin()->second.size() ;
+  for (unsigned int i = 0; i< N ; i++)
+    Fixed[bord.rbegin()->second[i]]=0.0;
+
+  //  printf("starting with %d vertices on the boundary\n",Fixed.size());
+
+  for (size_t i=0;i<tri.size();++i){
+    MVertex *v0 = tri[i]->getVertex(0);
+    MVertex *v1 = tri[i]->getVertex(1);
+    MVertex *v2 = tri[i]->getVertex(2);
+
+    std::map<MVertex*,double>::iterator it0 = Fixed.find(v0);
+    std::map<MVertex*,double>::iterator it1 = Fixed.find(v1);
+    std::map<MVertex*,double>::iterator it2 = Fixed.find(v2);
+
+    if (it0 != Fixed.end() && it1 != Fixed.end() && it2 == Fixed.end()){
+      //double d = computeDistanceLinePoint (v0,v1,v2);
+      double d = computeDistance (v0,0.0,v1,0.0,v2);
+      update(Close,v2,d);
+    }
+    else if (it0 != Fixed.end() && it2 != Fixed.end() && it1 == Fixed.end()){
+      //double d = computeDistanceLinePoint (v0,v2,v1);
+      double d = computeDistance (v0,0.0,v2,0.0,v1);
+      update(Close,v1,d);
+    }
+    else if (it2 != Fixed.end() && it1 != Fixed.end() && it0 == Fixed.end()){
+      //double d = computeDistanceLinePoint (v2,v1,v0);
+      double d = computeDistance (v2,0.0,v1,0.0,v0);
+      update(Close,v0,d);
+    }
+  }
+
+  //  printf("starting with %d vertices on the closed set\n",Close.size());
+  double CLOSEST = 0.0;
+  while(1){
+    if (Close.empty())break;
+    std::map<MVertex*,double>::iterator it = closest (Close);
+    CLOSEST = it->second;
+    //    printf ("CLOSEST = %lf\n",it->second);
+    Fixed[it->first]=it->second;
+    Close.erase(it);
+    std::vector<MElement*> &ts = v2t[it->first];
+    //    printf("%d elements around\n",ts.size());
+
+    for (unsigned int i=0;i<ts.size();i++){
+      MEdge ed = getEdge (ts[i],it->first);
+      std::map<MVertex*,double>::iterator it0 = Fixed.find(ed.getVertex(0));
+      std::map<MVertex*,double>::iterator it1 = Fixed.find(ed.getVertex(1));
+      //      printf("coucou %p %p\n",ed.getVertex(0),ed.getVertex(1));
+      if (it0 != Fixed.end() && it1 == Fixed.end()){
+	double d = computeDistance (it->first,it->second,it0->first,it0->second,ed.getVertex(1));
+	//	printf("neigh %d fixed 0 --> d = %g\n",i,d);
+	update(Close, ed.getVertex(1), d);
+      }
+      else if (it1 != Fixed.end() && it0 == Fixed.end()){
+	double d = computeDistance (it->first,it->second,it1->first,it1->second,ed.getVertex(0));
+	//	printf("neigh %d fixed 1 --> d = %g\n",i,d);
+	update(Close, ed.getVertex(0), d);
+      }
+    }
+    //    printf("%d %d\n",Fixed.size(),v2t.size());
+    if (Fixed.size() == v2t.size())break;
+  }
+
+  /*
+  char name[256];
+  sprintf(name,"geodesicDistance%d.pos",idNum);
+  FILE *f = fopen(name,"w");
+  fprintf(f,"View \"\"{\n");
+  for (unsigned int i=0;i<tri.size();i++){
+    double d0 = Fixed[tri[i]->getVertex(0)];
+    double d1 = Fixed[tri[i]->getVertex(1)];
+    double d2 = Fixed[tri[i]->getVertex(2)];
+    fprintf(f,"ST(%g,%g,%g,%g,%g,%g,%g,%g,%g){%g,%g,%g};\n",
+	    tri[i]->getVertex(0)->x(),tri[i]->getVertex(0)->y(),tri[i]->getVertex(0)->z(),
+	    tri[i]->getVertex(1)->x(),tri[i]->getVertex(1)->y(),tri[i]->getVertex(1)->z(),
+	    tri[i]->getVertex(2)->x(),tri[i]->getVertex(2)->y(),tri[i]->getVertex(2)->z(),d0,d1,d2);
+
+  }
+  fprintf(f,"};\n");
+  fclose(f);
+  */
+
+  return CLOSEST;
+}
+
+void discreteDiskFace::printAtlasMesh()
+{
+#if defined(HAVE_SOLVER) && defined(HAVE_ANN)
+  std::map<MVertex*,int> mv2int;
+  char buffer[256];
+  sprintf(buffer,"atlas_mesh%d.msh",initialTriangulation->idNum);
+  FILE* pmesh = Fopen(buffer,"w");
+	
+  std::set<MVertex*> meshvertices;
+	
+  for(unsigned int i=0; i<initialTriangulation->tri.size(); ++i){
+    MElement* tri = initialTriangulation->tri[i];
+    for(unsigned int j=0; j<3; j++)
+      if (meshvertices.find(tri->getVertex(j))==meshvertices.end()) meshvertices.insert(tri->getVertex(j));
+  }
+	
+  fprintf(pmesh,"$MeshFormat\n2.2 0 8\n$EndMeshFormat\n$Nodes\n%u\n",(unsigned int)meshvertices.size());
+  int count = 1;
+  for(std::set<MVertex*>::iterator it = meshvertices.begin(); it!=meshvertices.end(); ++it){
+    fprintf(pmesh,"%d %f %f %f\n",count,(*it)->x(),(*it)->y(),(*it)->z());
+    mv2int[*it] = count;
+    count++;
+  }
+  fprintf(pmesh,"$EndNodes\n$Elements\n%u\n",(unsigned int)initialTriangulation->tri.size());
+  for(unsigned int i=0; i<initialTriangulation->tri.size(); i++){
+    MElement* tri = initialTriangulation->tri[i];
+    fprintf(pmesh,"%d 2 2 0 0",i+1);
+    for(int j=0; j<3; j++){
+      MVertex* mv = tri->getVertex(j);
+      fprintf(pmesh," %d",mv2int[mv]);
+    }
+    fprintf(pmesh,"\n");
+  }
+  fprintf(pmesh,"$EndElements\n");
+  fclose(pmesh);
+#endif
+}
+
 void discreteDiskFace::printParamMesh()
 {
   std::map<MVertex*,int> mv2int;
-  char buffer[16];
-  sprintf(buffer,"param_mesh%d.msh",initialTriangulation->idNum);
-  FILE* pmesh = Fopen(buffer,"w");
+  char buffer[256];
+  sprintf(buffer,"param_mesh%d.msh",tag());
+  FILE* pmesh = fopen(buffer,"w");
   int count = 1;
   fprintf(pmesh,"$MeshFormat\n2.2 0 8\n$EndMeshFormat\n$Nodes\n%u\n",(unsigned int)allNodes.size());
   for(std::set<MVertex*>::iterator it = allNodes.begin(); it!=allNodes.end(); ++it){
@@ -901,8 +1327,8 @@ void discreteDiskFace::printParamMesh()
     MElement* tri = discrete_triangles[i];
     int p;
     _order == 1 ? p=2 : p=9;
-      fprintf(pmesh,"%d %d 2 0 0",i+1,p);
-    for(int j=0; j<_N; j++){
+    fprintf(pmesh,"%d %d 2 0 0",i+1,p);
+    for(int j=0; j<_n; j++){
       MVertex* mv = tri->getVertex(j);
       fprintf(pmesh," %d",mv2int[mv]);
     }
