@@ -22,6 +22,10 @@
 #include "meshGFaceOptimize.h"
 #include "PView.h"
 #include "robustPredicates.h"
+#include "MLine.h"
+#include "linearSystemCSR.h"
+#include "dofManager.h"
+
 
 inline int nodeLocalNum(MElement* e, MVertex* v)
 {
@@ -39,6 +43,15 @@ inline int edgeLocalNum(MElement* e, MEdge ed)
   return -1;
 }
 
+inline MEdge maxEdge(MElement* e)
+{
+  MEdge maxEd = e->getEdge(0);
+  for(int i=0; i<e->getNumEdges(); i++)
+    if (maxEd.length() < e->getEdge(i).length())
+      maxEd = e->getEdge(i);
+  return maxEd;
+}
+
 class ANNkd_tree;
 class Octree;
 class GRbf;
@@ -48,25 +61,32 @@ class triangulation {
  public:
 
   // attributes
+  int idNum; // number of identification, for hashing purposes
+  std::vector<MElement*> tri;// trianglse
+  GFace* gf;
+
+  double maxD;
   double area;
   bool seamPoint;
-  std::vector<MElement*> tri;// triangles
   std::set<MVertex*> vert;// nodes
   // edge to 1 or 2 triangle(s), their num into the vector of MElement*
   std::map<MEdge,std::vector<int>,Less_Edge> ed2tri;
   std::map<double,std::vector<MVertex*> > bord; //border(s)
   std::set<MEdge,Less_Edge> borderEdg; // border edges
-  GFace *gf;
-  int idNum; // number of identification, for hashing purposes
 
   std::list<GEdge*> my_GEdges;
+  std::set<int> fillingHoles;
 
   //---- methods
+  double geodesicDistance ();
 
   double aspectRatio()
-  { 
+  {
     double L = bord.rbegin()->first;
-    return L*L / (area*M_PI);
+    if (L == 0.0)return 1.e22;
+    if (maxD < 0) maxD = geodesicDistance();
+    //    printf("%12.5E %12.5E\n",L,maxD);
+    return maxD / L;
   }
 
   int genus()
@@ -117,7 +137,7 @@ class triangulation {
     std::map<MVertex*,std::vector<MVertex*> > firstNode2Edge;
     for (std::set<MEdge>::iterator ie = borderEdg.begin(); ie != borderEdg.end() ; ++ie) {
       MEdge ed = *ie;
-      std::vector<int> nT = ed2tri[ed];
+      const std::vector<int> &nT = ed2tri[ed];
       MElement* t = tri[nT[0]];
 
       std::vector<MVertex*> vecver;
@@ -129,7 +149,7 @@ class triangulation {
 
       std::map<MVertex*,std::vector<MVertex*> >::iterator im = firstNode2Edge.find(first);
       if (im != firstNode2Edge.end()){
-        Msg::Info("Supposed seam point for discreteFace %d", gf->tag());
+        Msg::Info("Assumed seam point for discreteFace %d", gf->tag());
 	seamPoint = true;
 	break;
       }
@@ -144,7 +164,7 @@ class triangulation {
       std::map<MVertex*,std::vector<MVertex*> >::iterator in = firstNode2Edge.begin();
       MVertex* previous = in->first;
       while(in != firstNode2Edge.end()) { // it didn't find it
-	std::vector<MVertex*> myV = in->second;
+	const std::vector<MVertex*> &myV = in->second;
 	for(unsigned int i=0; i<myV.size(); i++){
 	  loop.push_back(previous);
 	  MVertex* current = myV[i];
@@ -170,11 +190,22 @@ class triangulation {
     assignVert();
     assignEd2tri();
     assignBord();
+    maxD = -1.e22;
   }
 
+  void print (char *name, int elementary) const {
+    FILE *f = fopen (name,"w");
+    fprintf(f,"View \"\"{\n");
+    for (unsigned int i=0;i<tri.size();i++){
+      tri[i]->writePOS(f,true,false,false,false,false,false, 1.0, elementary);
+    }
+    fprintf(f,"};\n");
+    fclose(f);
+  }
+  
   triangulation() : gf(0) {}
-  triangulation(std::vector<MElement*> input, GFace* gface)
-    : tri(input), gf(gface) { assign(); }
+  triangulation(int id, const  std::vector<MElement*> &input, GFace* gface)
+    : idNum(id), tri(input), gf(gface){ assign(); }
 };
 
 // triangles in the physical space xyz, with their parametric coordinates
@@ -190,10 +221,11 @@ class  discreteDiskFaceTriangle {
 class discreteDiskFace : public GFace {
   GFace *_parent;
   void buildOct(std::vector<GFace*> *CAD = NULL) const;
-  bool parametrize(bool one2one = false) const;
+  bool parametrize(bool);// const;
+  void checklsys(linearSystemCSRTaucs<double>*,dofManager<double>*,int);
   void putOnView(bool,bool);
   bool checkOrientationUV();
-  void printParamMesh();
+  void optimize();
 
  public:
   discreteDiskFace(GFace *parent, triangulation* diskTriangulation,
@@ -213,18 +245,18 @@ class discreteDiskFace : public GFace {
   GPoint intersectionWithCircle(const SVector3 &n1, const SVector3 &n2,
 				const SVector3 &p, const double &d,
 				double uv[2]) const;
-
+  void printAtlasMesh () ;
+  void printParamMesh () ;
   
   std::vector<MElement*> discrete_triangles;
  protected:
-  // a copy of the mesh that should not be destroyed  
+  // a copy of the mesh that should not be destroyed
   triangulation* initialTriangulation;
   triangulation* geoTriangulation;// parametrized triangulation
-  
   std::vector<MVertex*> discrete_vertices;
 
   int _order;
-  int _N;// number of dof's for a triangle
+  int _n;// number of dof's for a triangle
   double _totLength;
   std::map<double,std::vector<MVertex*> > _loops;
   std::vector<MVertex*> _U0; // dirichlet's bc's
@@ -236,6 +268,7 @@ class discreteDiskFace : public GFace {
   mutable discreteDiskFaceTriangle *_ddft;
   mutable Octree *oct;
   mutable std::vector<double> _coords;
+
 };
 
 #endif
