@@ -6,13 +6,14 @@
 #ifndef BASISFACTORY_H
 #define BASISFACTORY_H
 
-#include "JacobianBasis.h"
-#include "FuncSpaceData.h"
+#include <map>
 class nodalBasis;
 class MetricBasis;
 class GradientBasis;
 class bezierBasis;
 class CondNumBasis;
+class JacobianBasis;
+class FuncSpaceData;
 
 class BasisFactory
 {
@@ -34,21 +35,8 @@ class BasisFactory
   // Warning: bases returned by BasisFactory::getJacobianBasis(int tag) are the
   // only safe bases for using Bezier on the jacobian determinant!
   static const JacobianBasis* getJacobianBasis(FuncSpaceData);
-  static const JacobianBasis* getJacobianBasis(int tag, int order) {
-    const int type = ElementType::ParentTypeFromTag(tag);
-    if (type != TYPE_PYR)
-      return getJacobianBasis(FuncSpaceData(true, tag, order));
-    else
-      return getJacobianBasis(FuncSpaceData(true, tag, false, order+1, order));
-  }
-  static const JacobianBasis* getJacobianBasis(int tag) {
-    const int order = JacobianBasis::jacobianOrder(tag);
-    const int type = ElementType::ParentTypeFromTag(tag);
-    if (type != TYPE_PYR)
-      return getJacobianBasis(FuncSpaceData(true, tag, order));
-    else
-      return getJacobianBasis(FuncSpaceData(true, tag, false, order+2, order));
-  }
+  static const JacobianBasis* getJacobianBasis(int tag, int order);
+  static const JacobianBasis* getJacobianBasis(int tag);
 
   // Metric
   static const MetricBasis* getMetricBasis(int tag);
@@ -58,22 +46,13 @@ class BasisFactory
 
   // Gradients
   static const GradientBasis* getGradientBasis(FuncSpaceData);
-  static const GradientBasis* getGradientBasis(int tag, int order) {
-    return getGradientBasis(FuncSpaceData(true, tag, order));
-  }
-  static const GradientBasis* getGradientBasis(int tag) {
-    return getGradientBasis(FuncSpaceData(tag));
-  }
+  static const GradientBasis* getGradientBasis(int tag, int order);
+  static const GradientBasis* getGradientBasis(int tag);
 
   // Bezier
   static const bezierBasis* getBezierBasis(FuncSpaceData);
-  static const bezierBasis* getBezierBasis(int parentTag, int order) {
-    int primaryTag = ElementType::getTag(parentTag, 1);
-    return getBezierBasis(FuncSpaceData(true, primaryTag, order));
-  }
-  static const bezierBasis* getBezierBasis(int tag) {
-    return getBezierBasis(FuncSpaceData(tag));
-  }
+  static const bezierBasis* getBezierBasis(int parentTag, int order);
+  static const bezierBasis* getBezierBasis(int tag);
 
   static void clearAll();
 };
