@@ -644,7 +644,12 @@ void HighOrderMeshOptimizer(GModel *gm, OptHomParameters &p)
     Msg::Info("Computing connectivity and bad elements for entity %d...",
               entity->tag());
     calcVertex2Elements(p.dim,entity,vertex2elements);
+
     if (p.optPrimSurfMesh) calcElement2Entity(entity,element2entity);
+    // Otherwise, still compute element2entity: Hack for using the geometry
+    // normal in the computation of the Jacobian when optimizing surface meshes.
+    // Warning: Accurate for planar surface but not really for curved surface...
+    else calcElement2Entity(entity,element2entity);
   }
 
   OptHomPeriodicity periodicity = OptHomPeriodicity(entities);
@@ -790,7 +795,7 @@ int HOPatchDefParameters::inPatch(const SPoint3 &badBary,
 }
 
 
-void HighOrderMeshOptimizerNew(GModel *gm, OptHomParameters &p)
+void HighOrderMeshOptimizerNew(std::vector<GEntity*> &entities, OptHomParameters &p)
 {
   Msg::StatusBar(true, "Optimizing high order mesh...");
 
@@ -849,7 +854,7 @@ void HighOrderMeshOptimizerNew(GModel *gm, OptHomParameters &p)
     par.pass.push_back(maxCADDistPass);
   }
 
-  meshOptimizer(gm, par);
+  meshOptimizer(entities, par);
 
   p.CPU = par.CPU;
   p.minJac = minMaxJacBarFunc.getMin();
@@ -857,3 +862,11 @@ void HighOrderMeshOptimizerNew(GModel *gm, OptHomParameters &p)
 
   Msg::StatusBar(true, "Done optimizing high order mesh (%g s)", p.CPU);
 }
+
+
+void HighOrderMeshOptimizerNew(GModel *gm, OptHomParameters &p)
+{
+  std::vector<GEntity*> entities;
+  gm->getEntities(entities);
+  HighOrderMeshOptimizerNew(entities, p);
+}
