@@ -45,8 +45,6 @@
   }
 }
 
--(bool) isReadOnly {return NO;}
-
 +(double)getHeight
 {
   return 60.0f;
@@ -58,12 +56,13 @@
 {
   self = [super init];
   if(self){
-    label.alpha = (string.getReadOnly())? 0.439216f : 1.0f;
     [label setText:[Utils getStringFromCString:string.getShortName().c_str()]];
     name = [Utils getStringFromCString:string.getName().c_str()];
     button = [UIButton buttonWithType:UIButtonTypeSystem];
     [button addTarget:self action:@selector(selectValue) forControlEvents:UIControlEventTouchDown];
     [button setTitle:[Utils getStringFromCString:string.getValue().c_str()] forState:UIControlStateNormal];
+    [button setEnabled:(string.getReadOnly() ? FALSE : TRUE)];
+    [label setEnabled:(string.getReadOnly() ? FALSE : TRUE)];
   }
   return self;
 }
@@ -78,7 +77,7 @@
                                 delegate:self cancelButtonTitle:nil destructiveButtonTitle:nil
                        otherButtonTitles:nil];
   std::vector<std::string> choices = string[0].getChoices();
-  for(int i=0;i<choices.size();i++)
+  for(int i = 0; i < choices.size(); i++)
     [popupSelectValue addButtonWithTitle:[Utils getStringFromCString:choices[i].c_str()]];
   [popupSelectValue addButtonWithTitle:@"Cancel"];
   [popupSelectValue setCancelButtonIndex:popupSelectValue.numberOfButtons - 1];
@@ -90,18 +89,22 @@
   std::vector<onelab::string> string;
   onelab::server::instance()->get(string, [name UTF8String]);
   if(string.size() < 1) return;
-  if(buttonIndex > string[0].getChoices().size() - 1) return; // cancel
-  std::string selected = string[0].getChoices()[buttonIndex];
-  string[0].setValue(selected);
-  onelab::server::instance()->set(string[0]);
-  [super editValue];
+  if(buttonIndex < string[0].getChoices().size()){
+    std::string selected = string[0].getChoices()[buttonIndex];
+    string[0].setValue(selected);
+    onelab::server::instance()->set(string[0]);
+    [super editValue];
+  }
 }
 
 -(void)refresh
 {
   std::vector<onelab::string> string;
   onelab::server::instance()->get(string, [name UTF8String]);
+  if(string.size() < 1) return;
   [button setTitle:[Utils getStringFromCString:string[0].getValue().c_str()] forState:UIControlStateNormal];
+  [button setEnabled:(string[0].getReadOnly() ? FALSE : TRUE)];
+  [label setEnabled:(string[0].getReadOnly() ? FALSE : TRUE)];
 }
 
 -(void)setFrame:(CGRect)frame
@@ -114,14 +117,6 @@
   return button;
 }
 
--(bool)isReadOnly
-{
-  std::vector<onelab::string> string;
-  onelab::server::instance()->get(string, [name UTF8String]);
-  if(string.size() < 1) return YES;
-  return string[0].getReadOnly();
-}
-
 +(double)getHeight
 {
   return 60.f;
@@ -133,13 +128,14 @@
 {
   self = [super init];
   if(self) {
-    label.alpha = (number.getReadOnly())? 0.439216f : 1.0f;
     [label setText:[Utils getStringFromCString:number.getShortName().c_str()]];
     name = [Utils getStringFromCString:number.getName().c_str()];
     button = [UIButton buttonWithType:UIButtonTypeSystem];
     [button addTarget:self action:@selector(selectValue) forControlEvents:UIControlEventTouchDown];
     [button setTitle:[Utils getStringFromCString:number.getValueLabel(number.getValue()).c_str()]
             forState:UIControlStateNormal];
+    [button setEnabled:(number.getReadOnly() ? FALSE : TRUE)];
+    [label setEnabled:(number.getReadOnly() ? FALSE : TRUE)];
   }
   return self;
 }
@@ -193,8 +189,11 @@
 {
   std::vector<onelab::number> number;
   onelab::server::instance()->get(number, [name UTF8String]);
+  if(number.size() < 1) return;
   [button setTitle:[Utils getStringFromCString:number[0].getValueLabel(number[0].getValue()).c_str()]
           forState:UIControlStateNormal];
+  [button setEnabled:(number[0].getReadOnly() ? FALSE : TRUE)];
+  [label setEnabled:(number[0].getReadOnly() ? FALSE : TRUE)];
 }
 
 -(void)setFrame:(CGRect)frame
@@ -207,14 +206,6 @@
   return button;
 }
 
--(bool)isReadOnly
-{
-  std::vector<onelab::number> number;
-  onelab::server::instance()->get(number, [name UTF8String]);
-  if(number.size() < 1) return YES;
-  return number[0].getReadOnly();
-}
-
 +(double)getHeight
 {
   return 60.f;
@@ -226,12 +217,13 @@
 {
   self = [super init];
   if(self) {
-    label.alpha = (number.getReadOnly())? 0.439216f : 1.0f;
     [label setText:[Utils getStringFromCString:number.getShortName().c_str()]];
     name = [Utils getStringFromCString:number.getName().c_str()];
     checkbox = [[UISwitch alloc] init];
     [checkbox setOn:(number.getValue() == 1)];
     [checkbox addTarget:self action:@selector(valueChange:) forControlEvents:UIControlEventValueChanged];
+    [checkbox setEnabled:(number.getReadOnly() ? FALSE : TRUE)];
+    [label setEnabled:(number.getReadOnly() ? FALSE : TRUE)];
   }
   return self;
 }
@@ -240,7 +232,10 @@
 {
   std::vector<onelab::number> number;
   onelab::server::instance()->get(number, [name UTF8String]);
+  if(number.size() < 1) return;
   [checkbox setSelected:(number[0].getValue() == 1)];
+  [checkbox setEnabled:(number[0].getReadOnly() ? FALSE : TRUE)];
+  [label setEnabled:(number[0].getReadOnly() ? FALSE : TRUE)];
 }
 
 -(void) valueChange:(UISwitch *)sender
@@ -263,14 +258,6 @@
   return checkbox;
 }
 
--(bool)isReadOnly
-{
-  std::vector<onelab::number> number;
-  onelab::server::instance()->get(number, [name UTF8String]);
-  if(number.size() < 1) return YES;
-  return number[0].getReadOnly();
-}
-
 +(double)getHeight
 {
   return 40.0f;
@@ -283,7 +270,6 @@
   self = [super init];
   if(self) {
     name = [Utils getStringFromCString:number.getName().c_str()];
-    label.alpha = (number.getReadOnly())? 0.439216f : 1.0f;
     stepper = [[UIStepper alloc] init];
     [stepper setValue:number.getValue()];
     [stepper setStepValue:1];
@@ -292,6 +278,8 @@
     [stepper addTarget:self action:@selector(stepperValueChanged:) forControlEvents:UIControlEventValueChanged];
     [label setText:[NSString stringWithFormat:@"%@ %d", [Utils getStringFromCString:number.getShortName().c_str()],
                              (int)number.getValue()]];
+    [stepper setEnabled:(number.getReadOnly() ? FALSE : TRUE)];
+    [label setEnabled:(number.getReadOnly() ? FALSE : TRUE)];
   }
   return self;
 }
@@ -316,6 +304,8 @@
   [stepper setValue:number[0].getValue()];
   [label setText:[NSString stringWithFormat:@"%@ %d", [Utils getStringFromCString:number[0].getShortName().c_str()],
                            (int)number[0].getValue()]];
+  [stepper setEnabled:(number[0].getReadOnly() ? FALSE : TRUE)];
+  [label setEnabled:(number[0].getReadOnly() ? FALSE : TRUE)];
 }
 
 -(void)setFrame:(CGRect)frame
@@ -339,7 +329,6 @@
 {
   self = [super init];
   if(self) {
-    label.alpha = (number.getReadOnly())? 0.439216f : 1.0f;
     name = [Utils getStringFromCString:number.getName().c_str()];
     slider = [[UISlider alloc] init];
     [slider setMaximumValue:number.getMax()];
@@ -350,6 +339,8 @@
     [slider addTarget:self action:@selector(sliderValueChanged:) forControlEvents:UIControlEventTouchUpInside];
     [label setText:[NSString stringWithFormat:@"%@ %g", [Utils getStringFromCString:number.getShortName().c_str()],
                              number.getValue()]];
+    [slider setEnabled:(number.getReadOnly() ? FALSE : TRUE)];
+    [label setEnabled:(number.getReadOnly() ? FALSE : TRUE)];
   }
   return self;
 }
@@ -364,6 +355,8 @@
   [slider setValue:number[0].getValue()];
   [label setText:[NSString stringWithFormat:@"%@ %g", [Utils getStringFromCString:number[0].getShortName().c_str()],
                            number[0].getValue()]];
+  [slider setEnabled:(number[0].getReadOnly() ? FALSE : TRUE)];
+  [label setEnabled:(number[0].getReadOnly() ? FALSE : TRUE)];
 }
 
 -(void)sliderValueChanged:(UISlider *)sender
@@ -388,14 +381,6 @@
   return slider;
 }
 
--(bool)isReadOnly
-{
-  std::vector<onelab::number> number;
-  onelab::server::instance()->get(number, [name UTF8String]);
-  if(number.size() < 1) return YES;
-  return number[0].getReadOnly();
-}
-
 +(double)getHeight
 {
   return 65.0f;
@@ -407,7 +392,6 @@
 {
   self = [super init];
   if(self) {
-    label.alpha = (number.getReadOnly())? 0.439216f : 1.0f;
     [label setText:[Utils getStringFromCString:number.getShortName().c_str()]];
     name = [Utils getStringFromCString:number.getName().c_str()];
     textbox = [[UITextField alloc] init];
@@ -421,6 +405,8 @@
                                    nil];
     [numberToolbar sizeToFit];
     textbox.inputAccessoryView = numberToolbar;
+    [textbox setEnabled:(number.getReadOnly() ? FALSE : TRUE)];
+    [label setEnabled:(number.getReadOnly() ? FALSE : TRUE)];
   }
   return self;
 }
@@ -431,6 +417,8 @@
   onelab::server::instance()->get(number, [name UTF8String]);
   if(number.size() < 1) return;
   [textbox setText:[NSString stringWithFormat:@"%g", number[0].getValue()]];
+  [textbox setEnabled:(number[0].getReadOnly() ? FALSE : TRUE)];
+  [label setEnabled:(number[0].getReadOnly() ? FALSE : TRUE)];
   [textbox reloadInputViews];
 }
 
@@ -466,14 +454,6 @@
   return textbox;
 }
 
--(bool)isReadOnly
-{
-  std::vector<onelab::number> number;
-  onelab::server::instance()->get(number, [name UTF8String]);
-  if(number.size() < 1) return YES;
-  return number[0].getReadOnly();
-}
-
 +(double)getHeight
 {
   return 60.f;
