@@ -52,7 +52,7 @@
                                       target:self action:@selector(backButtonPressed:)];
   }
   self.navigationItem.rightBarButtonItem =
-    [[UIBarButtonItem alloc] initWithTitle:@"Reset" style:UIBarButtonItemStyleBordered
+    [[UIBarButtonItem alloc] initWithTitle:@"Reset" style:UIBarButtonItemStylePlain
                                     target:self action:@selector(resetParameters:)];
 }
 
@@ -133,8 +133,7 @@
   }
   else if(p.getMin() >= p.getMax() ||
           p.getMin() == -onelab::number::maxNumber() ||
-          p.getMax() == onelab::number::maxNumber() ||
-          p.getReadOnly()) { // text box
+          p.getMax() == onelab::number::maxNumber()) { // text box
     ParameterNumberTextbox *param = [[ParameterNumberTextbox alloc] initWithNumber:p];
     [section addObject:param];
   }
@@ -216,6 +215,8 @@ NSString *GetSectionTitle(NSString *name)
   // check for new/updated parameters (number)
   for(int i = 0; i < number.size(); i++) {
     if(!number[i].getVisible()) continue; // do not add invisible parameter
+    if(number[i].getName() == "GetDP/}ModelCheck") continue; // don't show model checking
+
     NSString *name = [Utils getStringFromCString:number[i].getName().c_str()];
     NSString *sectiontitle = GetSectionTitle(name);
     Boolean found = false;
@@ -376,7 +377,6 @@ NSString *GetSectionTitle(NSString *name)
     cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier];
   }
   [cell setSelectionStyle:UITableViewCellSelectionStyleNone];
-  [cell setUserInteractionEnabled:!([tmp isReadOnly])];
   [tmp setLabelFrame:CGRectMake(20, 5, tableView.frame.size.width - 40, cell.frame.size.height/2)];
   [cell addSubview:[tmp getLabel]];
   if([tmp isKindOfClass:[ParameterStringList class]]) {
